## Supporting Information Material for gBartlett-type corrections and bootstrap adjustments of likelihood-based inference methods for network meta-analysish
## by Hisashi Noma, Kengo Nagashima, Kazushi Maruo and Toshi A. Furukawa
## Example R codes for implementing the developed inference methods.

# Data editing function to creating summary statistics

data.edit <- function(X1,X2){

	r <- is.na(X1[,1])
	X1[r,1] <- 0.001
	X2[r,1] <- 0.01

	N <- dim(X1)[1]
	p <- dim(X1)[2]

	y <- matrix(numeric(N*(p-1)),N)

	for(i in 2:p){

		z1 <- X1[,1]
		z2 <- X2[,1]

		x1 <- X1[,i]
		x2 <- X2[,i]

		y[,(i-1)] <- log(x1/(x2-x1)) - log(z1/(z2-z1)) 

	}

	S <- NULL

	for(i in 2:(p-1)){

		z1 <- X1[,1]
		z2 <- X2[,1]

		x1 <- X1[,i]
		x2 <- X2[,i]

		s11 <- 1/z1 + 1/(z2-z1) + 1/x1 + 1/(x2-x1)

		S <- cbind(S, s11)

		for(j in (i+1):p){

			w1 <- X1[,j]
			w2 <- X2[,j]

			s11 <- 1/z1 + 1/(z2-z1) + (x1 - x1) + (w1 - w1)    # NA̎ʂ̂߂ɁAŌ̍t^
			S <- cbind(S, s11)

		}

	}

	x1 <- X1[,p]
	x2 <- X2[,p]

	s11 <- 1/z1 + 1/(z2-z1) + 1/x1 + 1/(x2-x1)

	S <- cbind(S, s11); colnames(S) <- NULL

	mng2 <- list(y=y,S=S)

	return(mng2)

}

vmat <- function(q2){

 i1 <- 1; i2 <- p

 Sg <- matrix(numeric(p*p),p,p)

 for(i in 1:p){

  Sg[i,(i:p)] <- q2[i1:i2]

  i1 <- i2 + 1 
  i2 <- i2 + p - i

 }

 Sg <- Sg + t(Sg); diag(Sg) <- diag(Sg)/2

 return(Sg)

}

cmat <- function(q2){

 Sg <- q2*diag(p)

 return(Sg)

}

pmat <- function(Si, wi){

 pl <- length(wi)

 R <- matrix(numeric(pl*pl),pl)

 for(i in 1:pl){
  for(j in 1:pl){

   R[i,j] <- Si[wi[i],wi[j]]

  }
 }

 return(R)

}

imat <- function(Si, wi){

 pl <- length(wi)

 R <- matrix(numeric(p*p),p)

 for(i in 1:pl){
  for(j in 1:pl){

   R[wi[i],wi[j]] <- Si[i,j]

  }
 }

 return(R)

}

ivec <- function(yi, wi){

 pl <- length(wi)

 R <- numeric(p)
 
 for(i in 1:pl) R[wi[i]] <- yi[i]

 return(R)
 
}

ivec2 <- function(yi, wi){

 pl <- length(wi)

 R <- rep(NA,times=p)
 
 for(i in 1:pl) R[wi[i]] <- yi[i]

 return(R)
 
}

gmat <- function(g1,g2,p){

	G <- diag(0, p) + g2
	diag(G) <- g1
	return(G)

}










# ML algorithm for the consistent model (Sigma is exchangeble)

REML <- function(y,S,maxitr=200){

	N <- dim(y)[1]
	p <- dim(y)[2]

	mu <- rnorm(p)	# initial values
	g1 <- 0.2
	g2 <- 0.1
	
	Qc0 <- c(mu,g1,g2)
	
	LL1 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,(g/2),p)
		
		ll1 <- 0; XWX <- gmat(0,0,p)
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)

			XWX <- XWX + imat(B2,wi)
			
			ll1 <- ll1 + A1 + A2 + A3

		}

		ll2 <- ll1 + log(det(XWX)) - p*log(2*pi)
		
		return(ll2)
	
	}

	LL2 <- function(g){

		G <- gmat(g1,g,p)

		ll1 <- 0; XWX <- gmat(0,0,p)
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)

			XWX <- XWX + imat(B2,wi)
			
			ll1 <- ll1 + A1 + A2 + A3

		}

		ll2 <- ll1 + log(det(XWX)) - p*log(2*pi)
		
		return(ll2)
	
	}
		
	for(itr in 1:maxitr){
	
		A1 <- numeric(p)
		A2 <- matrix(numeric(p*p),p)
	
		G <- gmat(g1,g2,p)
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			Gi <- pmat(G,wi)

			Wi <- ginv(Gi + Si)
			
			A1 <- A1 + ivec(yi %*% Wi, wi)
			A2 <- A2 + imat(Wi, wi)

		}

		mu <- A1 %*% ginv(A2)
		g1 <- optimize(LL1, lower = 0, upper = 5)$minimum
		g2 <- 0.5*g1
		
		V.mu <- ginv(A2)
		
		Qc <- c(mu,g1,g2)

		rb <- abs(Qc - Qc0)/abs(Qc0); rb[is.nan(rb)] <- 0
		if(max(rb) < 10^-4) break

		Qc0 <- Qc
		
	}

	
	SE <- sqrt(diag(V.mu))

	R1 <- as.vector(mu)
	R2 <- as.vector(SE)
	R3 <- as.vector(mu - qnorm(.975)*SE)
	R4 <- as.vector(mu + qnorm(.975)*SE)

	R5 <- cbind(R1,R2,R3,R4); colnames(R5) <- c("Coef.","SE","95%CL","95%CU")

	R6 <- sqrt(g1)
	R7 <- g2/g1
	
	R8 <- list("Coefficients"=R5,"Between-studies_SD"=R6,"Between-studies_COR"=R7)
	
	return(R8)
	
}
	



# ML algorithm for the consistent model (Sigma is exchangeble)

ML <- function(y,S,maxitr=200){

	N <- dim(y)[1]
	p <- dim(y)[2]

	mu <- rnorm(p)	# initial values
	g1 <- 0.2
	g2 <- 0.1
	
	Qc0 <- c(mu,g1,g2)
	
	LL1 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,(g/2),p)
		
		ll1 <- 0
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)
	
	}

	LL2 <- function(g){

		G <- gmat(g1,g,p)

		ll1 <- 0
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)   # return "minus loglikelihood"
	
	}
		
	for(itr in 1:maxitr){
	
		A1 <- numeric(p)
		A2 <- matrix(numeric(p*p),p)
	
		G <- gmat(g1,g2,p)
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			Gi <- pmat(G,wi)

			Wi <- ginv(Gi + Si)
			
			A1 <- A1 + ivec(yi %*% Wi, wi)
			A2 <- A2 + imat(Wi, wi)

		}

		mu <- A1 %*% ginv(A2)
		g1 <- optimize(LL1, lower = 0, upper = 5)$minimum
		g2 <- 0.5*g1

		V.mu <- ginv(A2)

		Qc <- c(mu,g1,g2)

		rb <- abs(Qc - Qc0)/abs(Qc0); rb[is.nan(rb)] <- 0
		if(max(rb) < 10^-4) break
		if(itr==maxitr) print("Convergence criterion was not staisfied.")

		Qc0 <- Qc
		
	}



	SE <- sqrt(diag(V.mu))

	R1 <- as.vector(mu)
	R2 <- as.vector(SE)
	R3 <- as.vector(mu - qnorm(.975)*SE)
	R4 <- as.vector(mu + qnorm(.975)*SE)

	R5 <- cbind(R1,R2,R3,R4); colnames(R5) <- c("Coef.","SE","95%CL","95%CU")

	R6 <- sqrt(g1)
	R7 <- g2/g1
	
	R8 <- -LL1(g1)
	
	R9 <- list("Coefficients"=R5,"Between-studies_SD"=R6,"Between-studies_COR"=R7, "Loglikelihood"=R8)
	
	return(R9)
	
}


RMLP <- function(y,S,ml0,mu0,maxitr=200){

	N <- dim(y)[1]
	p <- dim(y)[2]

	muc <- ml0[2:p]		# initial values
	mu <- c(mu0,muc)
	g1 <- ml0[p+1]
	g2 <- ml0[p+2]
	
	Qc0 <- c(muc,g1,g2)

	RLL <- function(mur){

		mu1 <- c(mu0,mur)
		G <- gmat(g1,g2,p)

		ll1 <- 0
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu1[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + 0.5*(A1 + A2 + A3)

		}

		return(ll1)
	
	}
	
	LL1 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,g/2,p)

		ll1 <- 0
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)
	
	}

	LL2 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,g/2,p)

		ll1 <- 0
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)   # return "minus loglikelihood"
	
	}
		
	for(itr in 1:maxitr){
	
		A1 <- A2 <- A3 <- numeric(p-1)
		A4 <- matrix(numeric((p-1)*(p-1)),(p-1))
	
		G <- gmat(g1,g2,p)
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			Gi <- pmat(G,wi)

			Wi <- ginv(Gi + Si)
			
			Wi <- imat(Wi, wi)
			yi <- ivec(yi,wi)
			
			Wi21 <- Wi[2:p,1]
			Wi22 <- Wi[2:p,2:p]

			A1 <- A1 + Wi21 * yi[1]
			A2 <- A2 + Wi22 %*% yi[2:p]
			A3 <- A3 + Wi21 * mu0
			A4 <- A4 + Wi22
			
		}

		muc <- as.vector(A1 + A2 - A3) %*% ginv(A4)
		mu <- c(mu0,muc)

		g1 <- optimize(LL1, lower = 0, upper = 5)$minimum
		g2 <- 0.5*g1

		Qc <- c(muc,g1,g2)

		rb <- abs(Qc - Qc0)/abs(Qc0); rb[is.nan(rb)] <- 0
		if(max(rb) < 10^-4) break


		Qc0 <- Qc
		
	}

	R7 <- c(mu,g1,g2)
	R8 <- as.numeric(-RLL(muc))
	
	return(list("RML"=R7,"Loglikelihood"=R8))
	
}
	

	

ES <- function(y,S,ml0,mu0,maxitr=200){

	N <- dim(y)[1]
	p <- dim(y)[2]

	muc <- ml0[2:p]		# initial values
	mu <- c(mu0,muc)
	g1 <- ml0[p+1]
	g2 <- ml0[p+2]
	
	Qc0 <- c(muc,g1,g2)


	LL1 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,g/2,p)

		ll1 <- 0
		
		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)
	
	}

	LL2 <- function(g){

		#G <- gmat(g,g2,p)
		G <- gmat(g,g/2,p)

		ll1 <- 0
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			mui <- mu[wi]
			Gi <- pmat(G,wi)

			B1 <- (yi - mui)
			B2 <- ginv(Gi + Si)

			A1 <- log(det(Gi + Si))
			A2 <- t(B1) %*% B2 %*% B1
			A3 <- pl * log(2*pi)
		
			ll1 <- ll1 + A1 + A2 + A3

		}

		return(ll1)   # return "minus loglikelihood"
	
	}

	for(itr in 1:maxitr){
	
		A1 <- A2 <- A3 <- numeric(p-1)
		A4 <- matrix(numeric((p-1)*(p-1)),(p-1))
	
		G <- gmat(g1,g2,p)
		
		for(i in 1:N){
		
			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			pl <- length(wi)

			Si <- vmat(S[i,])

			yi <- yi[wi]
			Si <- pmat(Si,wi)
			Gi <- pmat(G,wi)

			Wi <- ginv(Gi + Si)
			
			Wi <- imat(Wi, wi)
			yi <- ivec(yi,wi)
			
			Wi21 <- Wi[2:p,1]
			Wi22 <- Wi[2:p,2:p]

			A1 <- A1 + Wi21 * yi[1]
			A2 <- A2 + Wi22 %*% yi[2:p]
			A3 <- A3 + Wi21 * mu0
			A4 <- A4 + Wi22
			
		}

		muc <- as.vector(A1 + A2 - A3) %*% ginv(A4)
		mu <- c(mu0,muc)

		g1 <- optimize(LL1, lower = 0, upper = 5)$minimum
		g2 <- 0.5*g1

		Qc <- c(muc,g1,g2)

		rb <- abs(Qc - Qc0)/abs(Qc0); rb[is.nan(rb)] <- 0
		if(max(rb) < 10^-4) break

		Qc0 <- Qc
		
	}

	A1 <- numeric(p)
	A2 <- matrix(numeric(p*p),p)
	
	G <- gmat(g1,g2,p)

	for(i in 1:N){
		
		yi <- as.vector(y[i,])
		wi <- which(is.na(yi)==FALSE)
		pl <- length(wi)

		Si <- vmat(S[i,])

		yi <- yi[wi]
		Si <- pmat(Si,wi)
		Gi <- pmat(G,wi)

		Wi <- ginv(Gi + Si)
			
		Wi <- imat(Wi, wi)
		yi <- ivec(yi,wi)
			
		A1 <- A1 + as.numeric(Wi %*% (yi - mu))
		A2 <- A2 + Wi
			
	}

	R1 <- as.numeric( t(A1) %*% ginv(A2) %*% A1 )
	
	return(R1)
	
}



	


PBS0 <- function(y,S,ml0,mu0,B=400){

	N <- dim(y)[1]
	p <- dim(y)[2]

	rml1 <- RMLP(y,S,ml0,mu0)
	G1 <- gmat(rml1$RML[p+1],rml1$RML[p+2],p)

	mu1 <- rml1$RML[1:p]

	y.pb <- matrix(numeric(N*p),N)

	# LR0.b <- numeric(B)
	ES.b <- numeric(B)
	
	for(b in 1:B){


		for(i in 1:N){

			yi <- as.vector(y[i,])
			wi <- which(is.na(yi)==FALSE)
			mSi <- vmat(S[i,]) + G1

			mui <- mu1[wi]
			mSi <- pmat(mSi,wi)

			y.pb[i,] <- ivec2(mvrnorm(1, mui, mSi),wi)

		}
		
		ES.b[b] <- ES(y.pb,S,ml0=ml0,mu0=mu0)
		
	}

	return(ES.b)
	
}


QT <- function(x,x0){

 x1 <- sort(c(x,x0))
 w1 <- which(x1==x0)
 qt <- 1 - w1/(length(x)+1)
 return(qt)
 
}

