## Supporting Information Material for “Bartlett-type corrections and bootstrap adjustments of likelihood-based inference methods for network meta-analysis”
## by Hisashi Noma, Kengo Nagashima, Kazushi Maruo and Toshi A. Furukawa
## Example R codes for implementing the developed inference methods.

library(MASS)

source(file="LRBSTS.r")

####

time0 <- Sys.time()

B1 <- 1600    # Number of bootstrap replication

dt1 <- read.csv(file="smoking.csv")

attach(dt1)

L <- dim(dt1)[1]

N <- max(study)
p <- max(trt)

X1 <- X2 <- matrix(rep(NA, times=N*p), N)

for(i in 1:L){

 k <- study[i]
 l <- trt[i]
 
 X1[k,l] <- d[i]
 X2[k,l] <- n[i]
 
}

###

p <- p - 1

for(i in 1:N){

 if(length(which(X1[i,]==0))>0){
  X1[i,] <- X1[i,] + 0.5
  X2[i,] <- X2[i,] + 1
 }

 if(length(which(X1[i,]==X2[i,]))>0){
  X1[i,] <- X1[i,] + 0.5
  X2[i,] <- X2[i,] + 1
 }

}

#####

X1[is.na(X1[,1]),1] <- 0.001
X2[is.na(X2[,1]),1] <- 0.01

####

de1 <- data.edit(X1,X2)
y <- de1$y
S <- de1$S

ml1 <- ML(y,S)

a1 <- ml1$Coefficients[,1]
a2 <- (ml1$`Between-studies_SD`)^2
a3 <- a2*(ml1$`Between-studies_COR`)

a4 <- c(a1,a2,a3)

mu13 <- ml1$Coefficients[1,1]
ci3 <- ml1$Coefficients[1,3:4]

R <- qchisq(0.95, df=1)

#####
#####

beta1e <- 0.79   # Computation of the lower limit of 95%CIs
C1 <- C2 <- NULL

repeat{

 beta1 <- log(beta1e)

 mlike0 <- .5*ml1$Loglikelihood
                                                                                                                                                                  
 bcb1 <- PBS0(y,S,ml0=a4,mu0=beta1,B=B1)

 mlike1 <- .5*RML(y,S,ml0=a4,mu0=beta1)                                                                                                                                                             
 LR0 <- -2*(mlike1 - mlike0)
 LR1 <- LR0/mean(bcb1)

 o1 <- LR0
 o2 <- LR1
 o3 <- mean(bcb1)
 o4 <- quantile(bcb1,0.95)
 o5 <- QT(bcb1,LR0)

 ####

 C1 <- rbind(C1, c(beta1e,o1,o2,o3,o4,o5))
 
 time1 <- Sys.time() - time0
 if((o2>R)&&(o1>o4)) break

 print(beta1e)
 beta1e <- beta1e - 0.01
 print(time1)

}

colnames(C1) <- c("OR","o1","o2","o3","o4","o5")
print(C1)

# OR: the parameter value (OR) for the null hypotheses
# o1: Likelihood ratio statistic
# o2: Bartlett-type corrected likelihood ratio statistic
# o3: Correcting factor for the Bartlett correction
# o4: Upper 5% quantile of the bootstrap distribution of the likelihood ratio statistic
# o5: P-value for the bootstrap test

